C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPAXES                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Plots axes of energy band graph.                   *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Changed for use with PGPLOT graphics library.      *
C *                                                                  *
C *  Called from: BANDPLOT,BPGRPH,BPRDRW                             *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSCI,PGSWIN,PGSVP,PGMOVE,PGDRAW   *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BPAXES(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,NSYM)
C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      INCLUDE 'sizes'
 
      DIMENSION VPXSCT(MAXSYM+1)
 
      MARGIN=0
C
C Select pen.
C
      IPEN=1
C
C NAG/GKS version.
C
C      CALL J06YMF(IPEN)
C
C PGPLOT version.
C
      CALL PGSCI(IPEN)
C
C Draw axes as simple lines.
C
      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH
C
C NAG/GKS version.
C
C      CALL J06WBF(0.0,VPXAXW,0.0,VPYAXH,MARGIN)
C      CALL J06WCF(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C PGPLOT version.
C
      CALL PGSWIN(0.0,VPXAXW,0.0,VPYAXH)
      CALL PGSVP(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C Draw x- and y-"axes".
C
C NAG/GKS version.
C
C      CALL J06YAF(0.0,VPYAXH)
C      CALL J06YCF(0.0,0.0)
C      CALL J06YCF(VPXAXW,0.0)
C
      CALL PGMOVE(0.0,VPYAXH)
      CALL PGDRAW(0.0,0.0)
      CALL PGDRAW(VPXAXW,0.0) 
C
C If more than one symmetry line is requested, draw a border and
C intersect the drawing region by vertical lines.
C
      IF (NSYM .GT. 1) THEN
C
C NAG/GKS version.
C
C        CALL J06YCF(VPXAXW,VPYAXH)
C        CALL J06YCF(0.0,VPYAXH)
C
C PGPLOT version.
C
        CALL PGDRAW(VPXAXW,VPYAXH)
        CALL PGDRAW(0.0,VPYAXH)
        DO I=2,NSYM
C
C NAG/GKS version.
C
C          CALL J06YAF(VPXSCT(I),0.0)
C          CALL J06YCF(VPXSCT(I),VPYAXH)
C
C PGPLOT version.
C
          CALL PGMOVE(VPXSCT(I),0.0)
          CALL PGDRAW(VPXSCT(I),VPYAXH)
        END DO
      ENDIF
C
C Use cheap trick for output of whole polyline buffer
C
C NAG/GKS version.
C
C      CALL J06YAF(0.0D0,0.0D0)
C 
      RETURN
      END