C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPRDRW                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Redraws band structure plot.                       *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group c/o                               *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 07.09.94                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Changed for use with PGPLOT graphics library.      *
C *                                                                  *
C *  Called from: BPGRPH                                             *
C *                                                                  *
C *  Calls......: BPAXES,BPBAND,BPBRDR,BPTICA,BPTRAN,BPFRMI          *
C *               PGPLOT routines PGSWIN,PGSVP                       *
C *                                                                  *
C ********************************************************************

      SUBROUTINE BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,
     .                  VPYEND,VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,
     .                  ETCMIN,ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .                  NSYM,ITICSD,ITXT,ITXPOS,
     .                  CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
      INCLUDE 'sizes'

      PARAMETER (MARGIN=0,ILEFT=0,ICENTR=1,IRIGHT=2)

      DIMENSION    VPXSCT(MAXSYM+1),ITICSD(MAXSYM+1),TEXTXY(MAXTEX,2)
      DIMENSION    TXTANG(MAXTEX),ITXPOS(MAXTEX)
      CHARACTER*80 CLABEL(MAXSYM+1),CXAXLB(MAXSYM),CYAXLB,CTEXT(MAXTEX),
     .             STRING
      LOGICAL      FBORDR,FFLINE,FSELEC(MAXSEL)


      CALL BPAXES(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,NSYM)
      CALL BPBAND(VPXOFF,VPYOFF,VPXSCT,VPYAXH,EMIN,EMAX,NSYM,IFAIL)

      IF (FSELEC(2)) THEN
        CALL BPBRDR(VPXOFF,VPYOFF,VPXAXW,VPYAXH,VPXEND,VPYEND)
      ENDIF
C
C Axes tic marks.
C
      IF (FSELEC(3)) THEN
C
C Annotate axis.
C
        INUM=1
        THRU=VPXOFF
        CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,ETCMAX,
     .              THRU,TICDIS,TICDS2,ITICSD(1),INUM)
C
C Step through each symmetry line, draw tic decoration.
C
        INUM=0
        IF (NSYM .GT. 1) THEN
          DO I=2,NSYM
            THRU=VPXOFF+VPXSCT(I)
            CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .                  ETCMAX,THRU,TICDIS,TICDS2,ITICSD(I),INUM)
          END DO
        ENDIF
        IF (FBORDR) THEN
          THRU=VPXEND
          CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .                ETCMAX,THRU,TICDIS,TICDS2,ITICSD(NSYM+1),INUM)
        ENDIF
C
C Use a cheap trick for output of whole polyline buffer.
C
C NAG/GKS version.
C
C        CALL J06YAF(1.0,EMIN)
      ENDIF
C
C Axes labels.
C
      IF (FSELEC(4)) THEN
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,1.0,0.0,1.0,MARGIN)
C        CALL J06WCF(0.0,1.0,0.0,1.0)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,1.0,0.0,1.0)
        CALL PGSVP(0.0,1.0,0.0,1.0)
        DO I=1,NSYM
          PENY=VPYOFF*0.75
          IF (I .EQ. 1) THEN
            PENX=VPXOFF
            STRING=CLABEL(1)
C
C NAG/GKS version.
C
C            CALL BPTRAN(STRING,PENX,PENY,ILEFT)
C
C PGPLOT version.
C
            CALL BPTRAP(STRING,PENX,PENY,0.0,ILEFT)
          ELSE
            PENX=VPXOFF+VPXSCT(I)
            STRING=CLABEL(I)
C
C NAG/GKS version.
C
C            CALL BPTRAN(STRING,PENX,PENY,ICENTR)
C
C PGPLOT version.
C
            CALL BPTRAP(STRING,PENX,PENY,0.0,ICENTR)
          ENDIF
        END DO
        PENX=VPXEND
        PENY=VPYOFF*0.75
        STRING=CLABEL(NSYM+1)
C
C NAG/GKS version.
C
C        CALL BPTRAN(STRING,PENX,PENY,IRIGHT)
C
C PGPLOT version.
C
        CALL BPTRAP(STRING,PENX,PENY,0.0,IRIGHT)
      ENDIF
C
C Symmetry line labels.
C
      IF (FSELEC(5)) THEN
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C        CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,VPXFUL,0.0,VPYFUL)
        CALL PGSVP(0.0,VPXFUL,0.0,VPYFUL)

        PENY=0.75*VPYOFF
        DO I=1,NSYM
          PENX=VPXOFF+(VPXSCT(I+1)+VPXSCT(I))/2.0
          STRING=CXAXLB(I)
C
C NAG/GKS version.
C
C          CALL BPTRAN(STRING,PENX,PENY,ICENTR)
C
C PGPLOT version.
C
          CALL BPTRAP(STRING,PENX,PENY,0.0,ICENTR)
        END DO

        STRING=CYAXLB
C
C NAG/GKS version.
C
C        PENX=0.1*VPXOFF
C        PENY=VPYEND*1.01
C        CALL BPTRAN(STRING,PENX,PENY,ILEFT)
C
C PGPLOT version.
C
        PENX=0.2*VPXOFF
        PENY=VPYOFF+0.5*VPYAXH
        CALL BPTRAP(STRING,PENX,PENY,90.,ICENTR)
      ENDIF
C
C Free text.
C
      IF (FSELEC(6)) THEN
C
C Set datarange and viewport.
C
C NAG/GKS version.
C 
C        CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C        CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,VPXFUL,0.0,VPYFUL)
        CALL PGSVP(0.0,VPXFUL,0.0,VPYFUL)
        DO I=1,ITXT
          STRING=CTEXT(I)
C
C NAG/GKS version.
C
C          CALL BPTRAN(STRING,TEXTXY(I,1),TEXTXY(I,2),ILEFT)
C
C PGPLOT version.
C
          CALL BPTRAP(STRING,TEXTXY(I,1),TEXTXY(I,2),TXTANG(I),
     .                ITXPOS(ITXT))
        END DO
      ENDIF
C
C Fermi level.
C
      IF (FSELEC(7)) THEN
        CALL BPFRMI(VPXOFF,VPYOFF,VPXFUL,VPYFUL,VPXAXW,VPYAXH,
     .              VPXEND,VPYEND,EMIN,EMAX,EFERMI,FFLINE)
      ENDIF
C
C NAG/GKS version.
C
C      CALL GUWK(1,1)
      RETURN
      END