C ********************************************************************
C * DOSAPP - Appends the data of two DOS/COOP-files                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DOSAPP                                                          *
C *                                                                  *
C *  Usage......: Appends the data of two DOS/COOP-files generated   *
C *               by EHPC. Useful if one wants to fit                *
C *               DOS curves with different gaussian lobes.          *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Institute f. Inorg. and Phys.    *
C *               Chemistry, University of Berne, Freiestr. 3,       *
C *               CH-3012 Berne                                      *
C *  Date.......: 08.01.93                                           *
C *  Revision...: -                                                  *
C *  Remarks....: Input:  Two files with DOS/COOP-data.              *
C *                       File 1: n1 energy mesh points,             *
C *                               energy range [E1..E2]              *
C *                       File 2: n2 energy mesh points,             *
C *                               energy range [E2..E3]              *
C *               Output: File 3: n1+n2-1 energy mesh points,        *
C *                               energy range [E1..E3]. The arith-  *
C *                               metic mean is taken from the data  *
C *                               at energy mesh point E2.           *
C *                                                                  *
C *               The program doesn't check the meaning of the data  *
C *               (description strings of AO/FMO-populations and     *
C *               COOP). It is therefore possible to generate non-   *
C *               sense. The description strings of file 1 are pun-  *
C *               ched into file 3.                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM DOSAPP

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      PARAMETER (NFILES=2,MXCARD=5,MXTITL=20)

      DIMENSION NDOS(NFILES),NPROJ(NFILES),NPROJF(NFILES),
     .          NPROJB(NFILES),NMESH(NFILES)
      CHARACTER CDUMMY*80

      IOUT=6
      IUNITO=IOUT+NFILES+1
      DO 20 I=1,NFILES
        IUNIT=IOUT+I
C
C MS-DOS version: add FILE=' ' to OPEN statement.
C
        OPEN(UNIT=IUNIT,ERR=970,STATUS='OLD',FILE=' ')
        READ(IUNIT,10) NDOS(I),NPROJ(I),NPROJF(I),NPROJB(I),NMESH(I)
   10   FORMAT(5I4)
   20 CONTINUE

      DO 50 I=2,NFILES
        IF ((NPROJ(I) .NE. NPROJ(1)) .OR. (NPROJF(I) .NE. NPROJF(1))
     .    .OR. (NPROJB(I) .NE. NPROJB(1)) .OR. (NDOS(I) .NE. NDOS(1)))
     .  THEN
          WRITE(IOUT,30)
   30     FORMAT(' Number of data sets not equal.')
          DO 40 J=1,NFILES
            IUNIT=IOUT+J
            CLOSE(IUNIT)
   40     CONTINUE
          STOP
        ENDIF
   50 CONTINUE

      NREAD=NDOS(1)+2*NPROJ(1)+2*NPROJF(1)+2*NPROJB(1)+1
      NMESHT=-NFILES+1
      DO 60 I=1,NFILES
        NMESHT=NMESHT+NMESH(I)
   60 CONTINUE
C
C Punch descriptor strings.
C
      OPEN(IUNITO,STATUS='NEW')
      WRITE(IUNITO,70) NDOS(1),NPROJ(1),NPROJF(1),NPROJB(1),NMESHT
      IUNITI=IOUT+1
   70 FORMAT(5I4)
      DO 90 I=1,3*MXCARD*MXTITL+3
        READ(IUNITI,80) CDUMMY
        WRITE(IUNITO,80) CDUMMY
        DO 75 J=2,NFILES
          IUNIT=IOUT+J
          READ(IUNIT,80) CDUMMY
   75   CONTINUE
   80   FORMAT(A80)

   90 CONTINUE
C
C Append the datasets.
C
      DO 130 I=1,NREAD
        DO 120 J=1,NFILES
          IUNIT=IOUT+J
          KSTART=1
          IF (J .NE. 1) THEN
            KSTART=2
            READ(IUNIT,100) VALUE2
            WRITE(IUNITO,100) (VALUE1+VALUE2)/2.0D0
          ENDIF
          KMESH=NMESH(J)
          DO 110 K=KSTART,KMESH
            READ(IUNIT,100) VALUE1
  100       FORMAT(F12.5)
            IF (J .LT. NFILES) THEN
              IF (K .LT. KMESH) THEN
                WRITE(IUNITO,100) VALUE1
              ENDIF
            ELSE
              WRITE(IUNITO,100) VALUE1
            ENDIF
  110     CONTINUE
  120   CONTINUE
  130 CONTINUE

      DO 140 I=1,NFILES+1
        IUNIT=IOUT+I
        CLOSE(IUNIT)
  140 CONTINUE
      STOP

  970 WRITE(IOUT,980) IUNIT
  980 FORMAT(' Input file error, Unit ',I2,'.')
      DO 990 J=1,I-1
        IUNIT=IOUT+J
        CLOSE(IUNIT)
  990 CONTINUE
      STOP
      END
