C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  CBORIS                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: SOLVES THE COMPLEX EIGENVALUE PROBLEM              *
C *               A*X=B*X*LAMBDA                                     *
C *                                                                  *
C *               WHERE                                              *
C *               A     --- GIVEN MATRIX OF ORDER N                  *
C *               B     --- GIVEN POSITIVE DEFINITE MATRIX OF ORDER N*
C *               X     --- EIGENVECTORS, COLUMN BY COLUMN,          *
C *                         NORMALIZED TO (BX,X)=I                   *
C *               LAMBDA --- EIGENVALUES IN INCREASING ORDER         *
C *                                                                  *
C *               MATRICES A AND B SHOULD BE GIVEN IN                *
C *               ARRAYS A AND B RESPECTIVELY IN THE FOLLOWING WAY   *
C *                                                                  *
C *               DIAGONAL ELEMENTS IN THE DIAGONAL                  *
C *               REAL PARTS OF THE LOWER TRIANGLE IN THE LOWER      *
C *               TRIANGLE                                           *
C *               IMAG PARTS OF THE LOWER TRIANGLE IN THE UPPER      *
C *               TRIANGLE                                           *
C *                                                                  *
C *               REAL PARTS OF EIGENVECTORS ARE STORED IN A,        *
C *               IMAG PARTS IN ARRAY C. ARRAY B IS DESTROYED DURING *
C *               COMPUTATION.  IT HOLDS THE LOWER TRIANGLE OF THE   *
C *               CHOLESKI DECOMPOSITION OF MATRIX B (SEE            *
C *               SUBROUTINE CCHOL).                                 *
C *                                                                  *
C *               ALL MATRICES ARE OF ORDER N, DECLARED IN THE       *
C *               CALLING PROGRAM WITH DIMENSION ND WHICH NEED       *
C *               NOT TO BE EQUAL TO N.                              *
C *                                                                  *
C *               EIGENVALUES ARE STORED IN ARRAY D.  ARRAYS E       *
C *               AND F ARE USED FOR INTERMEDIATE RESULTS.           *
C *                                                                  *
C *               IFAIL GETS THE FOLLOWING VALUES                    *
C *                0 --- COMPUTATION FINISHED SUCCESFULLY            *
C *                1 --- B NOT POSITIVE DEFINITE                     *
C *                2 --- QR ALGORITHM DOES NOT CONVERGE              *
C *                                                                  *
C *  Author.....: E. Zakrajsek, Annales de la Societe Scientifique   *
C *               de Bruxelles, T. 89(1975), 337                     *
C *  Date.......: 21.06.1974                                         *
C *  Revision...: -                                                  *
C *  Remarks....:                                                    *
C *                                                                  *
C *  Called from: CRYST,HUCKEL,EHFMO                                 *
C *                                                                  *
C *  Calls......: CCHOL,CTRED2,CTQL2                                 *
C *                                                                  *
C ********************************************************************

      SUBROUTINE CBORIS (N,ND,A,B,C,D,E,F,IFAIL)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      DIMENSION A(ND,ND),B(ND,ND),C(ND,ND),D(ND),E(ND),F(ND)
C
C DECOMPOSE MATRIX B
C
      IFAIL=1
      CALL CCHOL(N,ND,B,LF)
      IF (LF .NE. 0) RETURN 
C
C MOVE MATRIX A
C
      DO 20 I=1,N
        C(I,I)=0.0
        IF (I .EQ. 1) GO TO 20
        IA=I-1
        DO 10 J=1,IA
          C(I,J)=A(J,I)
          C(J,I)=-A(J,I)
   10   A(J,I)=A(I,J)
   20 CONTINUE
C
C  COMPUTE (L(-1)*A)
C
      DO 50 J=1,N
        DO 50 I=1,N
          IF (I .EQ. 1) GO TO 40
          IA=I-1
          DO 30 K=1,IA
            A(I,J)=A(I,J)-A(K,J)*B(I,K)+C(K,J)*B(K,I)
   30     C(I,J)=C(I,J)-A(K,J)*B(K,I)-C(K,J)*B(I,K)
   40     A(I,J)=A(I,J)/B(I,I)
   50 C(I,J)=C(I,J)/B(I,I)
C
C  COMPUTE  A*L(-H)
C
      DO 80 I=1,N
        DO 80 J=1,I
          IF (J .EQ. 1) GO TO 70
          JA=J-1
          DO 60 K=1,JA
            A(I,J)=A(I,J)-A(I,K)*B(J,K)-C(I,K)*B(K,J)
   60     C(I,J)=C(I,J)+A(I,K)*B(K,J)-C(I,K)*B(J,K)
   70     A(I,J)=A(I,J)/B(J,J)
   80 C(I,J)=C(I,J)/B(J,J)
C
C     PUT MATRIX TOGETHER INTO A
C
      DO 100 I=1,N
        IF (I .EQ. N) GO TO 100
        IA=I+1
        DO 90 J=IA,N
   90   A(I,J)=C(J,I)
  100 CONTINUE
C
C     DIAGONALIZE A
C
      IFAIL=2
      CALL CTRED2(N,ND,A,C,D,E,F)
      CALL CTQL2(N,ND,D,E,F,A,C,LF)
      IF (LF .NE. 0) RETURN 
C
C     COMPUTE L(-H)*A
C
      DO 130 J=1,N
        DO 130 II=1,N
          I=N-II+1
          IF (I .EQ. N) GO TO 120
          IA=I+1
          DO 110 K=IA,N
            A(I,J)=A(I,J)-A(K,J)*B(K,I)-C(K,J)*B(I,K)
  110     C(I,J)=C(I,J)+A(K,J)*B(I,K)-C(K,J)*B(K,I)
  120     A(I,J)=A(I,J)/B(I,I)
          C(I,J)=C(I,J)/B(I,I)
  130 CONTINUE
      IFAIL=0
 
      RETURN
      END
