C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  EHFMO                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: FMO analysis for molecules.                        *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 04.05.92/mbs (Martin Braendle c/o       *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012)                              *
C *  Remarks....: Slight modification of some COMMON blocks.         *
C *                                                                  *
C *  Called from: CRYST,HUCKEL                                       *
C *                                                                  *
C *  Calls......: CBORIS,ADDK,WRITEL,OMATT3,MATPAC,OUTMAT,OMFRAG     *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE EHFMO(DD,FF,V,VV,A,B,ND,NATOM)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      INTEGER      SORB(NATMMX)
      DIMENSION    DD(ND),FF(ND),V(ND,ND),VV(ND),A(ND,ND),B(ND,ND),
     .             ITEMP1(NDIMMX),ITEMP2(NDIMMX)
      CHARACTER*10 ITE(NDIMMX,2),ITE1(NDIMMX,2)
      CHARACTER*12 MOT
      CHARACTER*80 TEXT
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /FMOIO/IFMOIN(NDIMMX,NFMOMX),IFMOO(NDIMMX,NFMOMX),
     .              NDIMFR(NFMOMX),IFMOP(11,NFMOMX),NFMO
      COMMON /ORD/IPOSI(NDIMMX)
 
      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON /AOOUT2/INDT(NDIMMX,2)
      CHARACTER INDT*10
 
      IBAND=INDEX(KEYWRD,'BAND')
 
      DO 10 I=1,ND
        ITE1(I,1)=INDT(I,1)
        ITE1(I,2)=INDT(I,2)
   10 CONTINUE
 
      WRITE(IOUT,20)
   20 FORMAT(/1X,131('-')//,5X,'FMO analysis')
 
      DO 40 I=1,ND
        DO 30 J=1,ND
          A(I,J)=0.0D0
          B(I,J)=0.0D0
   30   CONTINUE
   40 CONTINUE
C
C   read Sij
C
      READ(21) (SORB(I),I=1,NATOM),V
      REWIND 21
C
C     block diagonalize S matrices --> S'
C
      DO 70 I=1,NFMO
        DO 60 J=1,ND
          DO 50 K=J,ND
            IF ((IFMOIN(J,I) .EQ. 1) .AND. (IFMOIN(K,I) .EQ. 1)) THEN
              B(K,J)=V(K,J)
            ENDIF
   50     CONTINUE
   60   CONTINUE
   70 CONTINUE
C
C read Hij
C
      READ(22) (SORB(I),I=1,NATOM),V
      REWIND 22
C
C     block diagonalize H matrices --> H'
C
      DO 100 I=1,NFMO
        DO 90 J=1,ND
          DO 80 K=J,ND
            IF ((IFMOIN(J,I) .EQ. 1) .AND. (IFMOIN(K,I) .EQ. 1)) THEN
              A(K,J)=V(K,J)
            ENDIF
   80     CONTINUE
   90   CONTINUE
  100 CONTINUE
C
C Solution of the polynomial equation !Hij-EkSij!=0
C H'-->a, S'-->b, v-->C', dd-->E'
C
      CALL CBORIS(ND,ND,A,B,V,DD,FF,VV,IFAIL)
      IF (IFAIL .NE. 0) THEN
        WRITE(IOUT,110) IFAIL
  110   FORMAT(10X,' IFAIL for MO = ',I3,' in EHFMO.')
        RETURN
      ENDIF
C
C test for degenaracy
C
      ITEST=0
      DELTA=1.0D-6
      DO 120 I=1,ND-1
        DDP=DD(I)+DELTA
        DDM=DD(I)-DELTA
        IF ((DD(I+1) .GT. DDM) .AND. (DD(I+1) .LT. DDP)) ITEST=1
  120 CONTINUE
      IF (ITEST .EQ. 1) THEN
        MOT='FORCE'
        CALL ADDK(MOT)
        TEXT='Warning : Degenerate system, results may be wrong!'
        CALL WRITEL(TEXT)
      ENDIF
C
C the eigenvalues E' and eigenvectors C' come out of the diagonalizer
C in order of increasing energy. This section orders them by fragment :
C
C  upon j
C
      NT=0
      DO 160 I=1,NFMO
        N=0
        DO 150 J=1,ND
          X=0.0D0
          DO 130 K=1,ND
            IF (IFMOIN(K,I) .EQ. 0) GOTO 130
            X=X+A(K,J)*A(K,J)
  130     CONTINUE
          IF (X .LT. 1.0D-6) GOTO 150
          N=N+1
          NT=NT+1
          IF (N .GT. NDIMFR(I)) THEN
            WRITE(IOUT,170) N,I
  170       FORMAT(///,' Error :  Ordering of eigenvalues in FMO n = ',
     .             I3,'   fragment = ',I3,///)
            RETURN
          ENDIF
          IF (NT .GT. ND) THEN
            WRITE(IOUT,180) NT
  180       FORMAT(///,' Error :  Ordering of eigenvalues in FMO n = ',
     .             I3,///)
            RETURN
          ENDIF
          FF(NT)=DD(J)
          DO 140 M=1,ND
            B(M,NT)=A(M,J)
  140     CONTINUE
  150   CONTINUE
  160 CONTINUE
C
C upon i
C
      N=0
      DO 230 I=1,NFMO
        DO 190 K=1,ND
          ITEMP1(K)=0
          ITEMP2(K)=0
  190   CONTINUE
        DO 210 J=1,ND
          IF (IFMOIN(J,I) .EQ. 1) THEN
            N=N+1
            ITEMP1(N)=IFMOO(J,I)
            ITEMP2(N)=IFMOIN(J,I)
            ITE(N,1)=INDT(J,1)
            ITE(N,2)=INDT(J,2)
            IPOSI(N)=J
            DO 200 K=1,ND
              V(N,K)=B(J,K)
  200       CONTINUE
          ENDIF
  210   CONTINUE
        DO 220 K=1,ND
          IFMOO(K,I)=ITEMP1(K)
          IFMOIN(K,I)=ITEMP2(K)
  220   CONTINUE
  230 CONTINUE
 
      DO 240 M=1,ND
        INDT(M,1)=ITE(M,1)
        INDT(M,2)=ITE(M,2)
  240 CONTINUE
C
C store C'
C
      WRITE(11) V
      REWIND 11
C
C output of E' and C'
C
      NT=0
      DO 320 I=1,NFMO
        IF (IFMOP(1,I) .EQ. -1) GOTO 280
        WRITE(IOUT,250) I
  250   FORMAT(//,5X,'Orbital energies (E'') of fragment # ',I2,/)
        M1=NT+IFMOP(1,I)
        M2=NT+IFMOP(2,I)
        IF (M1 .EQ. NT) M1=M1+1
        IF (M2 .EQ. NT) M2=M2+NDIMFR(I)
        DO 260 M=M1,M2
          MM=M-NT
  260   WRITE(IOUT,270) MM,FF(M)
  270   FORMAT(25X,I3,F12.6)
  280   IF (IFMOP(3,I) .EQ. -1) GOTO 310
 
        WRITE(IOUT,290) I
  290   FORMAT(//,5X,'Orbitals (C'') of fragment # ',I2,/)
        LB=NT+IFMOP(3,I)
        LE=NT+IFMOP(4,I)
        LBR=NT+IFMOP(5,I)
        LER=NT+IFMOP(6,I)
        IF (LB .EQ. NT) LB=LB+1
        IF (LE .EQ. NT) LE=LE+NDIMFR(I)
        IF (LBR .EQ. NT) LBR=LBR+1
        IF (LER .EQ. NT) LER=LER+NDIMFR(I)
        DO 300 K=1,ND
          ITEMP1(K)=IFMOO(K,I)
  300   CONTINUE
        MT=NT+1
        CALL OMATT3(FF,V,ND,LBR,LER,LB,LE,ITEMP1,MT)
 
  310   NT=NT+NDIMFR(I)
  320 CONTINUE
C
C options : MOFMO : T, FMOOVLP : S~, FMOINT : H~
C
      IMOFMO=0
      IFMOOV=0
      IFMOT=0
      DO 330 I=1,NFMO
        IFMOOV=IFMOOV+IFMOP(7,I)
        IFMOT=IFMOT+IFMOP(9,I)
        IMOFMO=IMOFMO+IFMOP(10,I)+IBAND
  330 CONTINUE
 
      IF ((IMOFMO .EQ. 0) .AND. (IFMOOV .LT. 2)) GOTO 360
C
C order and complete S
C
      READ(21) (SORB(I),I=1,NATOM),A
      REWIND 21
 
      DO 350 J=1,ND
        JOLD=IPOSI(J)
        DO 340 I=J,ND
          IOLD=IPOSI(I)
          B(I,J)=A(IOLD,JOLD)
          B(J,I)=A(IOLD,JOLD)
  340   CONTINUE
  350 CONTINUE
 
      IF (IBAND .NE. 0) GOTO 400
      WRITE(21) (SORB(I),I=1,NATOM),B
      REWIND 21
 
  360 IF (IMOFMO .EQ. 0) GOTO 390
C
C order and complete H
C
      READ(22) (SORB(I),I=1,NATOM), A
      REWIND 22
 
      DO 380 J=1,ND
        JOLD=IPOSI(J)
        DO 370 I=J,ND
          IOLD=IPOSI(I)
          B(I,J)=A(IOLD,JOLD)
          B(J,I)=A(IOLD,JOLD)
  370   CONTINUE
  380 CONTINUE
 
      WRITE(22) (SORB(I),I=1,NATOM),B
      REWIND 22
 
  390 IF (IMOFMO .EQ. 0) GOTO 460
C
C *********************************************************************
C MOFMO : T=C'^S'C   (C is read at the end of the subroutine)
C *********************************************************************
C
C read S
C
      READ(21) (SORB(I),I=1,NATOM),B
      REWIND 21
C
C calculate S'
C
  400 DO 410 I=1,ND
        DO 410 J=1,ND
          A(I,J)=0.0
  410 CONTINUE
 
      DO 440 I=1,NFMO
        DO 430 J=1,ND
          DO 420 K=J,ND
            IF ((IFMOIN(J,I) .EQ. 1) .AND. (IFMOIN(K,I) .EQ. 1)) THEN
              A(K,J)=B(K,J)
              A(J,K)=B(K,J)
            ENDIF
  420     CONTINUE
  430   CONTINUE
  440 CONTINUE
C
C calculate C'^S'
C
      CALL MATPAC(V,A,B,ND,ND,ND,ND,2)
C
C store C'^S'
C
      WRITE(7) B
      REWIND 7
      IF (IBAND .NE. 0) THEN
        DO 450 I=1,ND
          INDT(I,1)=ITE1(I,1)
          INDT(I,2)=ITE1(I,2)
  450   CONTINUE
        RETURN
      ENDIF
 
  460 IF (IFMOOV .LT. 2) GOTO 510
C
C *********************************************************************
C FMOOVLP : S~=C'^SC'
C *********************************************************************
C
C read S
C
      READ(21) (SORB(I),I=1,NATOM),A
      REWIND 21
C
C calculate C'^S
C
      CALL MATPAC(V,A,B,ND,ND,ND,ND,2)
C
C calculate C'^SC'=S~
C
      CALL MATPAC(B,V,A,ND,ND,ND,ND,1)
C
C print S~ between fragments
C
      II1=1
      DO 500 K=1,NFMO-1
        JJ1=II1+NDIMFR(K)
        DO 490 L=K+1,NFMO
 
          IF ((IFMOP(7,K) .EQ. 1) .AND. (IFMOP(7,L) .EQ. 1)) THEN
            LBR1=II1+IFMOP(5,K)-1
            LER1=II1+IFMOP(6,K)-1
            IF (LBR1 .EQ. II1-1) LBR1=LBR1+1
            IF (LER1 .EQ. II1-1) LER1=LER1+NDIMFR(K)
            LBR2=JJ1+IFMOP(5,L)-1
            LER2=JJ1+IFMOP(6,L)-1
            IF (LBR2 .EQ. JJ1-1) LBR2=LBR2+1
            IF (LER2 .EQ. JJ1-1) LER2=LER2+NDIMFR(L)
 
            WRITE(IOUT,470)
  470       FORMAT(//,5X,'FMO overlap (S~) between fragments ',/)
            WRITE(IOUT,480) K,L
  480       FORMAT(5X,'Rows : fragment # ',I2,', columns : fragment # ',
     .             I2,//)
 
            CALL OUTMAT(A,ND,LBR1,LER1,LBR2,LER2,II1,JJ1)
          ENDIF
 
          JJ1=JJ1+NDIMFR(L)
  490   CONTINUE
        II1=II1+NDIMFR(K)
  500 CONTINUE
 
  510 IF (IFMOT .LT. 2) GOTO 550
C
C *********************************************************************
C FMOINT : H~=C'^HC'
C *********************************************************************
C
C read H
C
      READ(22) (SORB(I),I=1,NATOM),A
      REWIND 22
C
C read C'
C
      READ(11)
      READ(11) V
      REWIND 11
C
C calculate C'^H
C
      CALL MATPAC(V,A,B,ND,ND,ND,ND,2)
C
C calculate C'^HC'=H~
C
      CALL MATPAC(B,V,A,ND,ND,ND,ND,1)
C
C print H~ between fragment
C
      II1=1
      DO 540 K=1,NFMO-1
        JJ1=II1+NDIMFR(K)
        DO 530 L=K+1,NFMO
          IF ((IFMOP(9,K) .EQ. 1) .AND. (IFMOP(9,L) .EQ. 1)) THEN
            LBR1=II1+IFMOP(5,K)-1
            LER1=II1+IFMOP(6,K)-1
            IF (LBR1 .EQ. II1-1) LBR1=LBR1+1
            IF (LER1 .EQ. II1-1) LER1=LER1+NDIMFR(K)
            LBR2=JJ1+IFMOP(5,L)-1
            LER2=JJ1+IFMOP(6,L)-1
            IF (LBR2 .EQ. JJ1-1) LBR2=LBR2+1
            IF (LER2 .EQ. JJ1-1) LER2=LER2+NDIMFR(L)
            WRITE(IOUT,520)
  520       FORMAT(//,5X,'FMO interaction energies (H~) between ',
     .             'fragments',/)
            WRITE(IOUT,480) K,L
            CALL OUTMAT(A,ND,LBR1,LER1,LBR2,LER2,II1,JJ1)
          ENDIF
          JJ1=JJ1+NDIMFR(L)
  530   CONTINUE
        II1=II1+NDIMFR(K)
  540 CONTINUE
 
  550 IF (IMOFMO .EQ. 0) RETURN
C
C *********************************************************************
C MOFMO : T=C'^S'C    (C=C'T)
C *********************************************************************
C
C order and complete C
C
      READ(11) B
      REWIND 11
 
      DO 570 J=1,ND
        DO 560 I=1,ND
          IOLD=IPOSI(I)
          A(I,J)=B(IOLD,J)
  560   CONTINUE
  570 CONTINUE
C
C
C read C'^S'
C
      READ(7) V
      REWIND 7
C
C calculate C'^S'C=T
C
      CALL MATPAC(V,A,B,ND,ND,ND,ND,1)
C
C print T
C
      WRITE(IOUT,580)
  580 FORMAT(//,5X,'MOs in terms of FMO (T)'/)
      CALL OMFRAG(B,ND)
 
      RETURN
      END
