C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  FTOCAR                                                          *
C *                                                                  *
C *  Subroutine of EHMACC                                            *
C *                                                                  *
C *  Usage......: Transforms fractional into cartesian coordinates.  *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Date.......: 11.11.94                                           *
C *  Available..: Calzaferri group, Institute for Inorganic and Phy- *
C *               sical chemistry, Freiestr. 3, CH-3012 Berne        *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: MATMUL                                             *
C *                                                                  *
C ********************************************************************

      SUBROUTINE FTOCAR(CZ)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION CZ(NATMMX,3),TRANSM(3,3),XIN(3,1),XOUT(3,1)

      COMMON /CELL/DIST(3,3)
      COMMON /FRACTC/FC(NATMMX,3)
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /LATCON/ALAT,BLAT,CLAT,ALPLAT,BETLAT,GAMLAT

      COMMON /ZMAT/BL(NATMMX),ALPH(NATMMX),BET(NATMMX),IZ(NATMMX,3),NZ

      COMMON /ZMATN/ANZ(NATMMX)
      CHARACTER ANZ*7
      
      PARAMETER(PI=3.14159265358979D0,RAD=PI/180.D0)
        

      WRITE(IOUT,10) ALAT,BLAT,CLAT,ALPLAT,BETLAT,GAMLAT
10    FORMAT(///,2X,'Lattice constants: ',6F10.4,/)

      ALPHA=ALPLAT*RAD
      BETA=BETLAT*RAD
      GAMMA=GAMLAT*RAD

      COSALP=COS(ALPHA)
      COSBET=COS(BETA)
      COSGAM=COS(GAMMA)
      SINGAM=SIN(GAMMA)

      V=1.0D0-COSALP*COSALP-COSBET*COSBET-COSGAM*COSGAM+
     .  2.0D0*COSALP*COSBET*COSGAM
      V=SQRT(V)

      TRANSM(1,1)=ALAT
      TRANSM(1,2)=BLAT*COSGAM
      TRANSM(1,3)=CLAT*COSBET
      TRANSM(2,1)=0.0D0
      TRANSM(2,2)=BLAT*SINGAM
      TRANSM(2,3)=CLAT*(COSALP-COSBET*COSGAM)/SINGAM
      TRANSM(3,1)=0.0D0
      TRANSM(3,2)=0.0D0
      TRANSM(3,3)=CLAT*V/SINGAM

      WRITE(IOUT,20)
20    FORMAT(/,2X,'Fractional coordinates:',//,
     .       4X,'Name',6X,'#',14X,'x',14X,'y',14X,'z',/)

      DO 60 I=1,NZ
        WRITE(IOUT,30) ANZ(I),I,(FC(I,J),J=1,3)
30      FORMAT(3X,A7,I5,3X,3F15.6)
        DO 40 J=1,3
          XIN(J,1)=FC(I,J)
40      CONTINUE
        CALL MATMUL(TRANSM,XIN,XOUT,3,3,3,1)
        DO 50 J=1,3
          CZ(I,J)=XOUT(J,1)
50      CONTINUE
60    CONTINUE
C
C Calculate the cell vectors.
C
      DO 90 I=1,3
        DO 70 J=1,3
          IF (I .EQ. J) THEN
            XIN(J,1)=1.0D0
          ELSE
            XIN(J,1)=0.0D0
          ENDIF
70      CONTINUE
        CALL MATMUL(TRANSM,XIN,XOUT,3,3,3,1)
        DO 80 J=1,3
          DIST(I,J)=XOUT(J,1)
80      CONTINUE
90    CONTINUE

      RETURN
      END
