C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GETKP                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Reads a set of wave vector k points for band cal-  *
C *               culations and options for analysis and output.     *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 04.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: PART1,CRYST                                        *
C *                                                                  *
C *  Calls......: WRITEL,READL,ADDK                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE GETKP (IEND)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'  

      CHARACTER   LINE*80,MOT*12
      CHARACTER*7 TEXT(MAXOPT+1)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /KPIN/TWEI,IKPOPT(MAXOPT+1),NKP

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2

      COMMON /SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),
     .               IPERMU(MXSYME,NATMMX,2),ISYMRE(MXSYME,NDIMMX),
     .               NLINOP(3),IPOINT(3,MXSYME),ILINE(3),NSYMEL,
     .               CSYMEL(MXSYME),FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2
C
C IEND=1: Called from PART1, check if k points exist and determine
C         sum TWEI of k point weights
C IEND=0: Called from CRYST, read in k point options
C
      ILEN=LEN(SLINE)
      IF (IEND .NE. 0) THEN
        NKP=0
        TWEI=0.0D0
        IF (INDEX(SLINE,'KP') .EQ. 0) THEN
          LINE='Cards KP expected.'
          CALL WRITEL(LINE)
          RETURN
        ENDIF

        WRITE(IOUT,10)
   10   FORMAT(' ')
      ENDIF

   20 IF (INDEX(SLINE,'KPIN') .NE. 0) THEN
        IF (IEND .NE. 0) THEN
          NKP=NKP+1
          IF (NKP .GT. NKMAX) THEN
            LINE='Number of k points limited to NKMAX.'
            CALL WRITEL(LINE)
            CALL CLEXIT
          ENDIF
        ENDIF

        XK(1)=READA(SLINE,ISTART(2),80)
        XK(2)=READA(SLINE,ISTART(3),80)
        XK(3)=READA(SLINE,ISTART(4),80)
        WEIGHT=0.0D0
C
C Clear pointers to symmetry operations for each symmetry line ILINE(I).
C
        IF (FSYM) THEN
          DO 40 I=1,3
            ILINE(I)=0
            NLINOP(I)=0
            DO 30 J=1,MXSYME
              IPOINT(I,J)=0
   30       CONTINUE
   40     CONTINUE
          DO 45 J=1,MXSYME
            FSYMAN(J)=.FALSE.
   45     CONTINUE    
        ENDIF
C
C Read k point weight, if it exists.
C
        IF (ISTART(5) .LE. 80) THEN
          MOT=SLINE(ISTART(5):ISTART(6)-2)
          IOFFST=5
          IF (INDEX(MOT,'LINE') .EQ. 0) THEN
            WEIGHT=READA(SLINE,ISTART(5),80)
            IOFFST=6
          ENDIF
          IF ((IEND .EQ. 0) .AND. (FSYM)) THEN
C
C Read symmetry line keyword and number of line.
C
            NLINE=0
   50       IF (ISTART(IOFFST) .GT. ILEN) GOTO 90
            MOT=SLINE(ISTART(IOFFST):ISTART(IOFFST+1)-2)
            IF (INDEX(MOT,'LINE') .EQ. 0) GOTO 90
            NLINE=NLINE+1
            IF (NLINE .GT. 3) GOTO 90
            ILINET=INT(READA(SLINE,ISTART(IOFFST),80))
C
C Read symmetry operations of symmetry line.
C
            NOP=0
   60       IOFFST=IOFFST+1
            IF (ISTART(IOFFST) .GT. ILEN) GOTO 90
            MOT=SLINE(ISTART(IOFFST):ISTART(IOFFST+1)-2)
            IF (INDEX(MOT,'OP') .EQ. 0) GOTO 50
            NOP=NOP+1
            IF (NOP .GT. MXSYME) THEN
              WRITE(IOUT,70) MXSYME
   70         FORMAT('  Maximum ',I2,' symmetry operations per line.')
              GOTO 90
            ENDIF
            IOP=INT(READA(SLINE,ISTART(IOFFST),80))
            IF ((IOP .GT. NSYMEL) .OR. ( .NOT. FSYMOP(IOP))) THEN
              WRITE(IOUT,80) IOP
   80         FORMAT('  Symmetry operation ',I2,' not defined.')
              NOP=NOP-1
            ELSE
              ILINE(NLINE)=ILINET
              IPOINT(NLINE,NOP)=IOP
              NLINOP(NLINE)=NOP
              FSYMAN(IOP)=.TRUE.
            ENDIF
            GOTO 60
   90       CONTINUE
          ENDIF
        ENDIF
        CALL READL

        DO 140 I=1,MAXOPT+1
          IKPOPT(I)=0
          TEXT(I)=' '
  140   CONTINUE
        IOPT=0

  150   IF (INDEX(SLINE,'KPOUT') .EQ. 0) GOTO 160
        IOPT=1
        IF (INDEX(SLINE,'COOUT') .NE. 0) THEN
          IKPOPT(1)=1
          TEXT(1)='COOUT'
          MOT='COOUT'
          IF (INDEX(KEYWRD,'COOUT') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF (INDEX(SLINE,' AOPOP') .NE. 0) THEN
          IKPOPT(2)=1
          TEXT(2)='AOPOP'
          MOT='AOPOP'
          IF (INDEX(KEYWRD,'AOPOP') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF (INDEX(SLINE,'RAOPOP') .NE. 0) THEN
          IKPOPT(3)=1
          TEXT(3)='RAOPOP'
          MOT='AOPOP'
          IF (INDEX(KEYWRD,'AOPOP') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF ((INDEX(SLINE,'COFMO') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'FMO') .NE. 0)) THEN
          IKPOPT(4)=1
          TEXT(4)='COFMO'
        ENDIF
        IF ((INDEX(SLINE,' FMOPOP') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'FMO') .NE. 0)) THEN
          IKPOPT(5)=1
          TEXT(5)='FMOPOP'
          MOT='FMOPOP'
          IF (INDEX(KEYWRD,'FMOPOP') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF ((INDEX(SLINE,'RFMOPOP') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'FMO') .NE. 0)) THEN
          IKPOPT(6)=1
          TEXT(6)='RFMOPOP'
          MOT='FMOPOP'
          IF (INDEX(KEYWRD,'FMOPOP') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF (INDEX(SLINE,'OVLPOP') .NE. 0) THEN
          IF (INDEX(KEYWRD,'OVLPOP') .EQ. 0) THEN
            SLINE='The keyword OVLPOP must be present.'
            CALL WRITEL (SLINE)
          ENDIF
          IKPOPT(7)=1
          TEXT(7)='OVLPOP'
        ENDIF
        IF ((INDEX(SLINE,'RFMOPOP') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'FMO') .NE. 0)) THEN
          IKPOPT(6)=1
          TEXT(6)='RFMOPOP'
          MOT='FMOPOP'
          IF (INDEX(KEYWRD,'FMOPOP') .EQ. 0) CALL ADDK(MOT)
        ENDIF
        IF ((INDEX(SLINE,'SKOUT') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'SKOUT') .NE. 0)) THEN
          IKPOPT(8)=1
          TEXT(8)='SKOUT'
        ENDIF     
        IF ((INDEX(SLINE,'HKOUT') .NE. 0) .AND. 
     .      (INDEX(KEYWRD,'HKOUT') .NE. 0)) THEN
          IKPOPT(9)=1
          TEXT(9)='HKOUT'
        ENDIF     
        CALL READL
        GOTO 150

  160   IF (IEND .EQ. 0) RETURN 
        TWEI=TWEI+WEIGHT
        NBL=0
        DO 190 I=1,MAXOPT
  170     IF (TEXT(I) .NE. ' ') GOTO 190
          DO 180 J=I,MAXOPT-NBL
            TEXT(J)=TEXT(J+1)
  180     CONTINUE
          TEXT(MAXOPT+1-NBL)=' '
          NBL=NBL+1
          NB=MAXOPT+1-NBL
          IF (NB .LE. I) GOTO 200
          GOTO 170
  190   CONTINUE
  200   IF (INDEX(KEYWRD,'CHECKFILE') .NE. 0) THEN
          WRITE(IOUT,210) NKP,(XK(I),I=1,3),WEIGHT
  210     FORMAT(2X,'KPIN  #',I3,2X,3F12.8,'        weight : ',G15.7)
          IF (IOPT .EQ. 1) WRITE(IOUT,220) NKP,(TEXT(I),I=1,MAXOPT)
  220     FORMAT(2X,'KPOUT #',I3,2X,8(A7,1X))
        ENDIF
        GOTO 20
      ENDIF

      WRITE(IOUT,10)
      IF (TWEI .LT. 1.0D-10) TWEI=DBLE(NKP)

      REWIND IIN
  230 CALL READL
      IF (INDEX(SLINE,'KPIN') .EQ. 0) GOTO 230

      RETURN
      END
