C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  ORBRD                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the orbital radii of the supplied ele-  *
C *               ments.                                             *
C *                                                                  *
C *  Author.....: Martin Braendle, c/o                               *
C *               Calzaferri group,                                  *
C *               Inst. f. Inorg. and Phys. Chemistry,               *
C *               University of Berne,                               *
C *               Freiestr. 3,                                       *
C *               CH-3012 Berne                                      *
C *  Date.......: 12.05.92                                           *
C *  Revision...: -                                                  *
C *  Remarks....: S. G.Calzaferri et al, J. Phys. Chem. 93(1989)5366 *
C *                                                                  *
C *  Called from: HMAT                                               *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE ORBRD

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),EXPD(NSYMMX),
     .              EXPD2(NSYMMX),EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              C(NATMMX,3)

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /AOOUT2/INDT(NDIMMX,2)
      CHARACTER INDT*10 

      COMMON /EQS/NAEQS(NATMMX)
      INTEGER NAEQS

      COMMON /ORBRAD/D0(NDIMMX)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      LOGICAL ORBOUT

      ORBOUT=(INDEX(KEYWRD,'ORBRADOUT') .GT. 0)

      IF (ORBOUT) WRITE (IOUT,10)
   10   FORMAT(//,'Orbital radii in distance dependent ',
     .           'Wolfsberg-Helmholz formula:',//,
     .           '  #   Atom       AO      c1     z1      c2     ',
     .           'z2       D0',/,
     .           '  -----------------------------------',
     .           '-----------------------')

      A0=0.52917706D0
      IORB=1
      DO 50 I=1,INATOM
        KEYI=ISYM(I)
        DO 15 KCOUNT=1,NAEQS(KEYI)
          IF (NS(KEYI,KCOUNT) .GT. 0) THEN
            IF (EXPS2(KEYI,KCOUNT) .EQ. 0.0D0) THEN
              D0(IORB)=DBLE(NS(KEYI,KCOUNT))/EXPS(KEYI,KCOUNT)*A0
            ELSE
              D01=CS1(KEYI,KCOUNT)*CS1(KEYI,KCOUNT)*EXPS(KEYI,KCOUNT)
              D02=CS2(KEYI,KCOUNT)*CS2(KEYI,KCOUNT)*EXPS2(KEYI,KCOUNT)
              N2=2*NS(KEYI,KCOUNT)
              D03=DBLE(2**N2)*(EXPS(KEYI,KCOUNT)*EXPS2(KEYI,KCOUNT))**
     .            (DBLE(NS(KEYI,KCOUNT)+0.5D0))
              D04=(EXPS(KEYI,KCOUNT)+EXPS2(KEYI,KCOUNT))**DBLE(N2)
              D05=D03/D04
              D0(IORB)=DBLE(NS(KEYI,KCOUNT))*A0/(D01+D02+D05)
            ENDIF

            IF (ORBOUT) THEN
              WRITE (IOUT, '(I3,2X,A7,3X,I1,A6,4F7.4,3X,F7.4)' ) I, 
     .               AC(I), NS(KEYI,KCOUNT), INDT(IORB,2), 
     .               CS1(KEYI,KCOUNT), EXPS(KEYI,KCOUNT), 
     .               CS2(KEYI,KCOUNT), EXPS2(KEYI,KCOUNT), D0(IORB) 
            ENDIF

            IORB=IORB+1
          ENDIF
   15   CONTINUE

        IF (NP(KEYI) .GT. 0) THEN
          DO 20 J=IORB,IORB+2
            IF (EXPP2(KEYI) .EQ. 0.0D0) THEN
              D0(J)=DBLE(NP(KEYI))/EXPP(KEYI)*A0
            ELSE
              D01=CP1(KEYI)*CP1(KEYI)*EXPP(KEYI)
              D02=CP2(KEYI)*CP2(KEYI)*EXPP2(KEYI)
              N2=2*NP(KEYI)
              D03=DBLE(2**N2)*(EXPP(KEYI)*EXPP2(KEYI))**
     .            (DBLE(NP(KEYI)+0.5D0))
              D04=(EXPP(KEYI)+EXPP2(KEYI))**DBLE(N2)
              D05=D03/D04
              D0(J)=DBLE(NP(KEYI))*A0/(D01+D02+D05)
            ENDIF

            IF (ORBOUT) THEN
              WRITE (IOUT, '(I3,2X,A7,3X,I1,A6,4F7.4,3X,F7.4)' ) I, 
     .               AC(I), NP(KEYI), INDT(J,2), CP1(KEYI), EXPP(KEYI), 
     .               CP2(KEYI), EXPP2(KEYI), D0(IORB) 
            ENDIF

20        CONTINUE
          IORB=IORB+3
        ENDIF

        IF (ND(KEYI) .GT. 0) THEN
          DO 30 J=IORB,IORB+4
            IF (EXPD2(KEYI) .EQ. 0.0D0) THEN
              D0(J)=DBLE(ND(KEYI))/EXPD(KEYI)*A0
            ELSE
              D01=CD1(KEYI)*CD1(KEYI)*EXPD(KEYI)
              D02=CD2(KEYI)*CD2(KEYI)*EXPD2(KEYI)
              N2=2*ND(KEYI)
              D03=DBLE(2**N2)*(EXPD(KEYI)*EXPD2(KEYI))**
     .            (DBLE(ND(KEYI)+0.5D0))
              D04=(EXPD(KEYI)+EXPD2(KEYI))**DBLE(N2)
              D05=D03/D04
              D0(J)=DBLE(ND(KEYI))*A0/(D01+D02+D05)
            ENDIF

            IF (ORBOUT) THEN
              WRITE (IOUT, '(I3,2X,A7,3X,I1,A6,4F7.4,3X,F7.4)' ) I, 
     .               AC(I), ND(KEYI), INDT(J,2), CD1(KEYI), 
     .               EXPD(KEYI), CD2(KEYI), EXPD2(KEYI), D0(IORB) 
            ENDIF

30        CONTINUE
          IORB=IORB+5
        ENDIF

        IF (NF(KEYI) .GT. 0) THEN
          DO 40 J=IORB,IORB+6
            IF (EXPF2(KEYI) .EQ. 0.0D0) THEN
              D0(J)=DBLE(NF(KEYI))/EXPF(KEYI)*A0
            ELSE
              D01=CF1(KEYI)*CF1(KEYI)*EXPF(KEYI)
              D02=CF2(KEYI)*CF2(KEYI)*EXPF2(KEYI)
              N2=2*NF(KEYI)
              D03=DBLE(2**N2)*(EXPF(KEYI)*EXPF2(KEYI))**
     .            (DBLE(NF(KEYI)+0.5D0))
              D04=(EXPF(KEYI)+EXPF2(KEYI))**DBLE(N2)
              D05=D03/D04
              D0(J)=DBLE(NF(KEYI))*A0/(D01+D02+D05)
            ENDIF

            IF (ORBOUT) THEN
              WRITE (IOUT, '(I3,2X,A7,3X,I1,A6,4F7.4,3X,F7.4)' ) I, 
     .               AC(I), NF(KEYI), INDT(J,2), CF1(KEYI), EXPF(KEYI), 
     .               CF2(KEYI), EXPF2(KEYI), D0(IORB) 
            ENDIF

40        CONTINUE
          IORB=IORB+7
        ENDIF
50    CONTINUE
      
      RETURN
      END
