C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SMOOTH                                                          *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Applies gaussian smoothing to generate the total   *
C *               energy, the density of states, and the integrated  *
C *               density of states as a function of energy (for a   *
C *               set of given mesh points) for plotting purposes.   *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 31.03.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Formatted output will be generated.                *
C *                                                                  *
C *  Called from: PPART                                              *
C *                                                                  *
C *  Calls......: FDST,WRITEL,OUTPUT                                 *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SMOOTH(ENDK,W,EDX,XND,XNDI,TEDI,XNORM,ID,NDK)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION ENDK(NDK),W(NDK),EDX(NMESH),XND(NMESH),XNDI(NMESH),
     .          TEDI(NMESH),ID(NDK)
C
C 14.10.93/mbs
C 
C Passed parameters:
C
C ENDK : energy levels (#=NDK)
C ID   : sorted pointers to energy levels (ID(1)=lowest,ID(NDK)=highest)
C W    : weigths for each energy level
C NDK  : total number of energy levels (#kpoints*#AOs)
C
C Returned parameters:
C
C XNORM: 
C
C Passed array space:
C 
C EDX  : Energy mesh 
C XND  : Smoothed density of states
C XNDI : Smoothed integrated DOS
C TEDI : Smoothed total energy
C
      COMMON /ENERG/IENERG
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/ KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /OPTV/DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .             IVARYE,NVARY
      COMMON /ORB2/NATOM,IND(NDIMMX),NORBEF(NATMMX),NORBE(NATMMX),NDEL

C
C Initialization
C
      IF (EMIN .LT. ENDK(ID(1))) EMIN=ENDK(ID(1))
      IF (EMAX .GT. ENDK(ID(NDK))) EMAX=ENDK(ID(NDK))
      IENERG=0
      XSTEP=(EMAX-EMIN)/DBLE((NMESH-1))
      DO 10 I=1,NMESH
        EDX(I)=EMIN+XSTEP*DBLE(I-1)
        XND(I)=0.0D0
        XNDI(I)=0.0D0
        TEDI(I)=0.0D0
   10 CONTINUE
C
C Look for the last point
C
      IRUN=NMESH
      N3=NDK
      N1=1
   30 N2=(N3+N1)/2
      IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 40
      IF (ENDK(ID(N2)) .GT. EDX(IRUN)) THEN
        N3=N2
      ELSE
        N1=N2
      ENDIF
      GOTO 30
   40 NEND=N2
      IF (ENDK(ID(N2)) .GT. EDX(IRUN)) NEND=NEND-1
C
C Look for the first point
C
      IRUN=1
      N3=NDK
      N1=1
   50 N2=(N3+N1)/2
      IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 60
      IF (ENDK(ID(N2)) .GT. EDX(IRUN)) THEN
        N3=N2
      ELSE
        N1=N2
      ENDIF
      GOTO 50
   60 NSTART=N2
      IF (ENDK(ID(N2)) .LT. EDX(IRUN)) NSTART=NSTART+1
      NST=NSTART

      IF (DELTA .EQ. 0.0D0) DELTA=PDELTA
      IF (XLIMIT .EQ. 0.0D0) XLIMIT=PLIMIT
      ELIM=-LOG(XLIMIT)
      ELIM=SQRT(ELIM)*DELTA
C
C XNADDB: Tail area of DOS gaussian below EMIN  
C XNADDE: Tail area of DOS gaussian above EMAX
C XEADDB: Tail area of total energy gaussian below EMIN
C XEADDE: Tail area of total energy gaussian above EMAX
C
      XNADDB=0.0D0
      XNADDE=0.0D0
      XEADDB=0.0D0
      XEADDE=0.0D0
C
C Fit gaussian slopes onto energy mesh.
C IRUN runs through all energy mesh points.
C
      IRUN=0
   70 IRUN=IRUN+1
      IF (IRUN .GT. NMESH) GOTO 120
      CVAL=EDX(IRUN)
      IF (IRUN .GT. 1) CALL FDST(ENDK,ID,NST,CVAL,NDK)
      NCOU=NST
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than 
C ELIM. Right k-space part of energy level CVAL.  
C
   80 I1=ID(NCOU)
      DIF=ENDK(I1)-CVAL
      ABSD=ABS(DIF)
      IF (ABSD .GT. ELIM) GOTO 90
      Z=DIF/DELTA
      Z=Z*Z
      ZE=DEXP(-Z)*W(I1)*2.0D0
      XND(IRUN)=XND(IRUN)+ZE
      IF (NCOU .GT. NEND) THEN
        XNADDE=XNADDE+ZE
        XEADDE=XEADDE+CVAL*ZE
      ENDIF
      NCOU=NCOU+1
      IF (NCOU .LE. NDK) GOTO 80
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than 
C ELIM. Left k-space part of energy level CVAL.  
C
   90 NCOU=NST-1
  100 IF (NCOU .LE. 0) GOTO 110
      I1=ID(NCOU)
      DIF=ABS(ENDK(I1)-CVAL)
      IF (DIF .GT. ELIM) GOTO 110
      Z=DIF/DELTA
      Z=Z*Z
      ZE=DEXP(-Z)*W(I1)*2.0D0
      XND(IRUN)=XND(IRUN)+ZE
      IF (NCOU .LT. NSTART) THEN
        XNADDB=XNADDB+ZE
        XEADDB=XEADDB+CVAL*ZE
      ENDIF
      NCOU=NCOU-1
      GOTO 100
  110 GOTO 70
  120 CONTINUE
C
C XNLTB: DOS below EMIN
C XNLTE: DOS above EMAX
C XELTB: Total energy below EMIN
C XELTE: Total energy above EMAX
C
      XNLTB=0.0D0
      XNLTE=0.0D0
      XELTB=0.0D0
      XELTE=0.0D0
C
C Determine DOS and total EHMO energy below EMIN by the above procedure
C
      NCOU=-1
      XRUN=EDX(1)
  130 XRUN=XRUN-XSTEP
      IF (NCOU .EQ. NSTART) GOTO 150
      NCOU=NSTART
  140 I1=ID(NCOU)
      DIF=ABS(ENDK(I1)-XRUN)
      IF (DIF .GT. ELIM) GOTO 130
      Z=DIF/DELTA
      Z=Z*Z
      ZE=DEXP(-Z)*W(I1)*2.0D0
      XNLTB=XNLTB+ZE
      XELTB=XELTB+XRUN*ZE
      NCOU=NCOU+1
      IF (NCOU .GT. NDK) GOTO 130
      GOTO 140
  150 CONTINUE
C
C Determine DOS and total EHMO energy below EMIN by the above procedure
C
      NCOU=-1
      XRUN=EDX(NMESH)
  160 XRUN=XRUN+XSTEP
      IF (NCOU .EQ. NEND) GOTO 180
      NCOU=NEND
  170 I1=ID(NCOU)
      DIF=ABS(ENDK(I1)-XRUN)
      IF (DIF .GT. ELIM) GOTO 160
      Z=DIF/DELTA
      Z=Z*Z
      ZE=DEXP(-Z)*W(I1)*2.0D0
      XNLTE=XNLTE+ZE
      XELTE=XELTE+XRUN*ZE
      NCOU=NCOU-1
      IF (NCOU .LT. 1) GOTO 160
      GOTO 170
  180 CONTINUE

      COEFF=0.0D0
      DO 190 I=1,NMESH
        COEFF=COEFF+XND(I)
  190 CONTINUE

      COEFF=COEFF+XNLTB+XNLTE-XNADDB-XNADDE
      COEFF=COEFF*XSTEP

      XELEC=0.0D0
      DO 200 I=NSTART,NEND
        XELEC=XELEC+W(ID(I))
  200 CONTINUE
      XELEC=XELEC*2.0D0

      XNORM=XELEC/COEFF
C
C Check against theoretical normal.
C
      TNORM=1.0D0/(DELTA*SQRT(3.141592654D0))
      CDIF=DABS(XNORM-TNORM)/TNORM
      IF (CDIF .GT. 1.D-4) THEN
        SLINE='Normalisation error > 0.1% : too restrictive limitation,'
     .         //' to go ahead use the FORCE option.'
        CALL WRITEL(SLINE)
      ENDIF

      IF (INDEX(KEYWRD,'SDOS') .EQ. 0) RETURN

      DO 210 I=1,NMESH
        XND(I)=XND(I)*XNORM
  210 CONTINUE

      XBE=0.0D0
      XTE=0.0D0
      DO 220 I=1,NSTART-1
        I1=ID(I)
        XBE=XBE+W(I1)*2.0D0
        XTE=XTE+W(I1)*2.0D0*ENDK(I1)
  220 CONTINUE
      XBE=XBE+(XNLTB-XNADDB)*XNORM*XSTEP
      XTE=XTE+(XELTB-XEADDB)*XNORM*XSTEP

      DO 230 I=1,NMESH
        XBE=XBE+XSTEP*XND(I)
        XTE=XTE+XSTEP*XND(I)*EDX(I)
        XNDI(I)=XBE
        TEDI(I)=XTE
  230 CONTINUE

      WRITE(7,240) EDX
      WRITE(7,240) XND
      WRITE(7,240) XNDI
      WRITE(7,240) TEDI
  240 FORMAT(F12.5)

      TMID=(TEDI(1)+TEDI(NMESH))*0.5D0

      DO 250 I=1,NMESH
        TEDI(I)=TEDI(I)-TMID
  250 CONTINUE
C
C Output of energy mesh points
C
      IENERG=1
      WRITE(IOUT,260)
  260 FORMAT(//////,5X,'Energy mesh points',//)
      CALL OUTPUT(EDX,NMESH)
C
C Output of DOS
C
      WRITE(IOUT,270)
  270 FORMAT(//////,5X,'Smoothed density of states',//)
      CALL OUTPUT(XND,NMESH)
C
C Output of IDOS
C
      WRITE(IOUT,280)
  280 FORMAT(//////,5X,'Smoothed integrated density of states',//)
      CALL OUTPUT(XNDI,NMESH)
C
C Output of total energy
C
      WRITE(IOUT,290) TMID
  290 FORMAT(//////,5X,'Smoothed total energy = ',F15.3,'  +  ',//)
      CALL OUTPUT(TEDI,NMESH)

      RETURN
      END
