C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BCC                                                             *
C *                                                                  *
C *  Usage......: Generates special k point set after A.H. McDonald, *
C *               Phys. Rev. B 18(1978), 5897 for the bcc direct     *
C *               lattice (fcc reciprocal).                          *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Inst. f. Inorganic and Physical  *
C *               Chemistry, University of Berne, Freiestr. 3,       *
C *               CH-3012 Berne                                      *
C *  Date.......: 30.03.93/mbs                                       *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM BCC

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING
      LOGICAL L1,L2,L3

      WRITE(6,10)
10    FORMAT(' k point set after A.H. McDonald, Phys. Rev. B 18(1978),',
     .       ' 5897,',/,' for the bcc direct lattice (fcc reciprocal).',
     .       /)
20    WRITE(6,30)
30    FORMAT(' Enter q: ',$)
      READ(5,40,ERR=20) IQ
40    FORMAT(I3)
      IF (IQ .LT. 1) GOTO 20

      IF (DBLE(IQ)/2.D0 .NE. DBLE(IQ/2)) THEN
        N=(IQ+1)*(IQ+3)*(IQ+5)
      ELSE IF (DBLE(IQ)/4.0D0 .EQ. DBLE(IQ/4)) THEN
        N=IQ*(IQ*IQ+9*IQ+44)
      ELSE
        ITEST=IQ/8
        ITEST=IQ-ITEST*8
        IF (ITEST .EQ. 2) THEN
          N=IQ*IQ*IQ+9*IQ*IQ+38*IQ+24
        ELSE
          N=IQ*IQ*IQ+9*IQ*IQ+50*IQ-72
        ENDIF
      ENDIF
      N=N/48

      WRITE(6,50) N
50    FORMAT(' Approxinate number of k points: ',I4)

      WRITE(6,60)
60    FORMAT(' Enter optional string for output options: ',$)
      READ(5,70) STRING
70    FORMAT(A80)

      WRITE(1,75) N
75    FORMAT('!',/,'! bcc direct lattice, approx. ',I3,' k-points,',/,
     .       '!  according to A.H. McDonald, Phys. Rev. B 18(1978)',/,
     .       '!')

      N=0
      Q=DBLE(IQ)
      QBOUND=1.5D0*Q
      DO 120 NX=0,IQ
        DO 110 NY=0,NX
          DO 100 NZ=0,NY
            IF (DBLE(NX+NY+NZ) .LE. QBOUND) THEN
              L1=(DBLE(NX/2) .EQ. DBLE(NX)/2.0D0)
              L2=(DBLE(NY/2) .EQ. DBLE(NY)/2.0D0)
              L3=(DBLE(NZ/2) .EQ. DBLE(NZ)/2.0D0)
              IF ((L1 .AND. L2 .AND. L3) .OR. ((.NOT. L1) .AND. (.NOT.
     .             L2) .AND. (.NOT. L3))) THEN
C
C Determine weights of k points.
C
C General point.
C
                NWEIGT=48

                IF ((NX .EQ. NY) .AND. (NY .EQ. NZ)) THEN
                  IF (NX .EQ. 0) THEN
C
C Gamma point.
C
                    NWEIGT=1
                  ELSE IF (2*NX .EQ. IQ) THEN
C
C P point.
C
                    NWEIGT=2
                  ELSE
C
C Lambda axis.
C
                    NWEIGT=8
                  ENDIF
                ELSE IF (NZ .EQ. 0) THEN
                  IF (NY .EQ. 0) THEN
                    IF (NX .EQ. IQ) THEN
C
C H point
C
                      NWEIGT=1
                    ELSE
C
C Delta axis.
C
                      NWEIGT=6
                    ENDIF
                  ELSE
                    IF (NY .EQ. NX) THEN
                      IF (2*NX .EQ. IQ) THEN
C
C N point.
C
                        NWEIGT=6
                      ELSE
C
C Sigma axis.
C
                        NWEIGT=12
                      ENDIF
                    ELSE
                      IF (IQ/2-NY .EQ. IQ/2+NX) THEN
C
C G line.
C
                        NWEIGT=12
                      ELSE
C
C Gamma-H-N plane.
C
                        NWEIGT=24
                      ENDIF
                    ENDIF
                  ENDIF
                ELSE
                  IF ((2*NX .EQ. IQ) .AND. (2*NY .EQ. IQ)) THEN
C
C D line.
C
                    NWEIGT=12
                  ELSE IF ((NZ .EQ. NY) .AND. (IQ/2-NY .EQ. IQ/2+NX))
     .            THEN
C
C F line.
C
                    NWEIGT=8
                  ELSE IF ((NY+NX .EQ. IQ) .OR. ((NZ .EQ. NX) .AND.
     .                     (NX .GT. NZ)) .OR. ((NY .EQ. NZ) .AND.
     .                     (NZ .LT. NX))) THEN
C
C H-N-P, Gamma-N-P, Gamma-H-P planes.
C
                    NWEIGT=24
                  ELSE
                  ENDIF
                ENDIF

                WRITE(1,80) DBLE(NX)/Q,DBLE(NY)/Q,DBLE(NZ)/Q,NWEIGT
80              FORMAT('KPIN ',3F13.10,1X,I2)
                N=N+1
                IF (STRING(1:1) .GT. ' ') THEN
                  WRITE(1,90) STRING
90                FORMAT(A80)
                ENDIF
              ENDIF
            ENDIF
100       CONTINUE
110     CONTINUE
120   CONTINUE
      WRITE(6,130) N
130   FORMAT(' Exact number of k points: ',I4)

      STOP
      END
