C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  FCC                                                             *
C *                                                                  *
C *  Usage......: Generates special k point set after A.H. McDonald, *
C *               Phys. Rev. B 18(1978), 5897 for the fcc direct     *
C *               lattice (bcc reciprocal).                          *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Inst. f. Inorganic and Physical  *
C *               Chemistry, University of Berne, Freiestr. 3,       *
C *               CH-3012 Berne                                      *
C *  Date.......: 25.06.93/mbs                                       *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM FCC

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING
      LOGICAL LEVEN,LEVEN2

      WRITE(6,10)
10    FORMAT(' k point set after A.H. McDonald, Phys. Rev. B 18(1978),',
     .       ' 5897,',/,' for the fcc direct lattice (bcc reciprocal).',
     .       /)
20    WRITE(6,30)
30    FORMAT(' Enter q: ',$)
      READ(5,40,ERR=20) IQ
40    FORMAT(I3)
      IF (IQ .LT. 1) GOTO 20

      IF (2*(IQ/2) .NE. IQ) THEN
        LEVEN=.FALSE.
        N=(IQ+1)*(IQ+3)*(IQ+5)
      ELSE IF (4*(IQ/4) .EQ. IQ) THEN
        LEVEN=.TRUE.
        N=IQ*(IQ+4)*(IQ+5)
      ELSE
        LEVEN=.TRUE.
        N=(IQ+1)*(IQ+2)*(IQ+6)
      ENDIF
      N=N/48

      WRITE(6,50) N
50    FORMAT(' Approxinate number of k points: ',I4)

      WRITE(6,60)
60    FORMAT(' Enter optional string for output options: ',$)
      READ(5,70) STRING
70    FORMAT(A80)

      WRITE(1,75) N
75    FORMAT('!',/,'! fcc direct lattice, approx. ',I3,' k-points,',/,
     .       '!  according to A.H. McDonald, Phys. Rev. B 18(1978)',/,
     .       '!')

      N=0
      Q=DBLE(IQ)

      DO 120 NX=0,IQ
        DO 110 NY=0,NX
          DO 100 NZ=0,NY
            IF ((NX+NY) .LE. IQ) THEN
              LEVEN2=(2*((NX+NY+NZ)/2) .EQ. (NX+NY+NZ))
              IF ((LEVEN .AND. .NOT. LEVEN2) .OR.
     .            (.NOT. LEVEN .AND. LEVEN2)) THEN
C
C Determine weights of k points.
C
C General point.
C
                NWEIGT=48
                IF ((NX .EQ. NY) .AND. (NY .EQ. NZ)) THEN
                  IF (NX .EQ. 0) THEN
C
C Gamma point.
C
                    NWEIGT=1
                  ELSE IF (2*NX .EQ. IQ) THEN
C
C L point.
C
                    NWEIGT=4
                  ELSE
C
C LAMBDA line.
C
                    NWEIGT=8
                  ENDIF
                ELSE IF (NX .EQ. NY) THEN
                  IF (NZ .EQ. 0) THEN
C
C SIGMA line and K point.
C
                    NWEIGT=12
                  ELSE
C
C GAMMA-K-L plane.
C
                    NWEIGT=24
                  ENDIF
                ELSE IF ((NZ .EQ. NY) .AND. (NZ .LT. NX)) THEN
C
C GAMMA-L-U-X plane
C
                  NWEIGT=24
                  IF (NZ .EQ. 0) THEN
C
C DELTA line.
C
                    NWEIGT=6
C
C X point.
C
                    IF (NX .EQ. IQ) NWEIGT=3
                  ELSE IF (NX .EQ. IQ) THEN
C
C S line and U point.
C
                    NWEIGT=12
                  ELSE
                  ENDIF
                ELSE IF (NX .EQ. IQ) THEN
C
C U-W-X plane.
C
                  NWEIGT=24
                  IF (NZ .EQ. 0) THEN
C
C Z line.
C
                    NWEIGT=12
                    IF (NY .EQ. IQ/2) THEN
C
C W point.
C
                      NWEIGT=6
                    ENDIF
                  ENDIF
                ELSE IF ((NY .EQ. IQ/2) .AND. (IQ/2-NZ .EQ. IQ/2+NX))
     .          THEN
C
C Q line.
C
                  NWEIGT=24
                ELSE IF (NZ .EQ. 0) THEN
C
C GAMMA-K-W-X plane.
C
                  NWEIGT=24
                ELSE
                ENDIF
                WRITE(1,80) DBLE(NX)/Q,DBLE(NY)/Q,DBLE(NZ)/Q,NWEIGT
80              FORMAT('KPIN ',3F13.10,1X,I2)
                N=N+1
                IF (STRING(1:1) .GT. ' ') THEN
                  WRITE(1,90) STRING
90                FORMAT(A80)
                ENDIF
              ENDIF
            ENDIF
100       CONTINUE
110     CONTINUE
120   CONTINUE
      WRITE(6,130) N
130   FORMAT(' Exact number of k points: ',I4)

      STOP
      END
