C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  HEXAG2                                                          *
C *                                                                  *
C *  Usage......: Generates k-point mesh of hexagonal lattice, grid  *
C *               type 2b, according to R. Ramirez, M.C. Boehm,      *
C *               Int. J. Quant. Chem. 30 (1986) 391                 *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 27.12.93/mbs                                       *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM HEXAG2

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING

      WRITE(6,10)
10    FORMAT(/,' Generate k-point mesh (hexagonal lattice, type 2b)',/)
20    WRITE(6,30)
30    FORMAT(' Number of k-points along x direction, total is ',
     .      '(N*N+N)/2: ',$)
      READ(5,40,ERR=20) N
40    FORMAT(I3)
      IF (N .LT. 1) GOTO 20

      WRITE(6,50)
50    FORMAT(' Enter optional string for output options: ',$)
      READ(5,60) STRING
60    FORMAT(A80)

      NKMAX=(N*N+N)/2

      DX=0.3333333333D0/DBLE(N)
      DY=DX
      NCOUNT=3*N/2+1
      X=0.0D0
      NTOTW=0

      WRITE(1,70) NKMAX
70    FORMAT('!',/,
     .       '! Hexagonal lattice, grid type 2b, ',I3,' k-points,',/,
     .       '! according to R. Ramirez, M.C. Boehm, Int. J. Quant. '
     .       'Chem. 30 (1986), 391',/,'!')

      
      DO 110 I=1,N
        NWEIGT=2
        Y=DBLE(I-1)*DY
        IF (I .EQ. 1) NWEIGT=1
        X=Y
        DO 100 J=1,NCOUNT
          X=X+DX
          IF (3*INT(J/3) .NE. J) THEN
            IF (Y .LE. (1.0D0-2.0D0*X)) THEN
              WRITE(1,80) X,Y,0.0D0,NWEIGT
80            FORMAT('KPIN ',3F13.10,1X,I2)
              IF (STRING(1:1) .GT. ' ') THEN
                WRITE(1,90) STRING
90              FORMAT(A80)
              ENDIF
              NTOTW=NTOTW+NWEIGT
            ENDIF
          ENDIF
100     CONTINUE
110   CONTINUE 

      WRITE(6,120) DBLE(NTOTW)/DBLE(NKMAX*2)
120   FORMAT(' Efficiency of grid: ',F7.4)
   
      STOP
      END
